#include <net-snmp/net-snmp-config.h>
#include <net-snmp/utilities.h>
#include <net-snmp/net-snmp-includes.h>
#include <getopt.h>
#include <string.h>
#include <time.h>



void getIF_snmpgetinterface(struct snmp_session *ss,int printresult){
  struct snmp_pdu *pdu;
  struct snmp_pdu *response;
  struct variable_list *vars=NULL;
  int status;
  oid      sOID[MAX_OID_LEN];
  size_t sOID_len = MAX_OID_LEN;  
  oid      anOID[MAX_OID_LEN];
  size_t anOID_len = MAX_OID_LEN;  
  char buf[255]="mib-2.ip.ipAddrTable";
  int  buflen=255;
  int count=0;
  int ipAddrEntrys=3;          /* the table has 5 columns but the first is parsed to get the number of entrys, ipAdEntReasmMaxSize seems not to be implemented on the test-maschine */ 
  int ifEntrys=8;
  int ifNumber=0;
  sOID_len=MAX_OID_LEN;
  

  /* GET THE INTERFACE-Specific CONTENT */


  /* FETCH NUMBER OF ROWS */
  pdu = snmp_pdu_create(SNMP_MSG_GET);
  get_node("interfaces.ifNumber.0",anOID,&anOID_len);
  snmp_add_null_var(pdu, anOID, anOID_len);  
  status = snmp_synch_response(ss, pdu, &response);
  /* Process the response. */
  if (status == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR) {
    if (printresult)
      for(vars = response->variables; vars; vars = vars->next_variable)
	print_variable(vars->name, vars->name_length, vars);
      vars = response->variables;
      ifNumber=(int) *vars->val.integer;
  }
  pdu = snmp_pdu_create(SNMP_MSG_GETBULK);
  /* GET TABLE */
  pdu->non_repeaters = 0;
  pdu->max_repetitions = ifEntrys*ifNumber;     /* Each Row has 16 entrys multiplied with the number of interfaces  */
  anOID_len=MAX_OID_LEN;
  get_node("mib-2.interfaces.ifTable.ifEntry.ifIndex",anOID,&anOID_len);
  snmp_add_null_var(pdu, anOID, anOID_len);
  status = snmp_synch_response(ss, pdu, &response);
  if (status == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR) {
    if (printresult)
      for(vars = response->variables; vars; vars = vars->next_variable)
	print_variable(vars->name, vars->name_length, vars);    
  } else {
    if (status == STAT_SUCCESS)
      fprintf(stderr, "Error in packet\nReason: %s\n",
	      snmp_errstring(response->errstat));
    else
      snmp_sess_perror("snmpget", ss);
  }  
  if (response)
    snmp_free_pdu(response);

  
  /** GET THE IP-Specific CONTENT */
  do{
    /* FETCH THE FIRST INDEXES OF THE ROWS */
    pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
    sOID_len=MAX_OID_LEN;
    get_node(buf,sOID,&sOID_len);
    snmp_add_null_var(pdu, sOID, sOID_len);  
    status = snmp_synch_response(ss, pdu, &response);
    /* Process the response. */
    if (status == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR) {
      if (printresult)
	for(vars = response->variables; vars; vars = vars->next_variable)
	  print_variable(vars->name, vars->name_length, vars);
      
      vars = response->variables;   
      if (vars->type == ASN_IPADDRESS) {
	//u_char         *ip = vars->val.string;
	//printf("FOUND INDEX %d.%d.%d.%d\n", ip[0], ip[1], ip[2],ip[3]);	
	count++;
	bzero(buf,buflen);
	snprint_objid (buf,buflen,vars->name,vars->name_length);
	//printf("OID= %s\n",buf);
      }
    }
    if (response)
      snmp_free_pdu(response);
  }while (vars->type == ASN_IPADDRESS);

  ipAddrEntrys*=count;        /* GET THE NUMBER OF ENTRYS TO GET */
  pdu = snmp_pdu_create(SNMP_MSG_GETBULK);

  /* GET TABLE */
  pdu->non_repeaters = 0;
  pdu->max_repetitions = ipAddrEntrys;     
  sOID_len=MAX_OID_LEN;
  get_node(buf,sOID,&sOID_len);
  snmp_add_null_var(pdu, sOID, sOID_len);

  /*
   * Send the Request out.
   */
  status = snmp_synch_response(ss, pdu, &response);
  
  /*
   * Process the response.
   */
  if (status == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR) {
    /*
     * SUCCESS: Print the result variables
     */
    if (printresult)
      for(vars = response->variables; vars; vars = vars->next_variable)
	print_variable(vars->name, vars->name_length, vars);    
  } else {
    /*
     * FAILURE: print what went wrong!
     */
    if (status == STAT_SUCCESS)
      fprintf(stderr, "Error in packet\nReason: %s\n",
	      snmp_errstring(response->errstat));
    else
      snmp_sess_perror("snmpget", ss);
  }  
  if (response)
    snmp_free_pdu(response);


}




void getIF_snmpgetroute(struct snmp_session *ss, int printresult){
  struct snmp_pdu *pdu;
  struct snmp_pdu *response;
  struct variable_list *vars=NULL;
  int status;
  oid      sOID[MAX_OID_LEN];
  size_t sOID_len = MAX_OID_LEN;  
  char buf[255]="mib-2.ip.ipRouteTable";
  int  buflen=255;
  int count=0;
  int ipRouteEntrys=7;          /* the table has 13 columns but the first is parsed to get the number of entrys, ipRouteMetric2,3,4,5 ,ipRouteAge are absent  */ 
  sOID_len=MAX_OID_LEN;
  

  

  
  /** GET THE IP-ROUTE-Specific CONTENT */
  do{
    /* FETCH THE FIRST INDEXES OF THE ROWS */
    pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
    sOID_len=MAX_OID_LEN;
    get_node(buf,sOID,&sOID_len);
    snmp_add_null_var(pdu, sOID, sOID_len);  
    status = snmp_synch_response(ss, pdu, &response);
    /* Process the response. */
    if (status == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR) {
      if (printresult)
	for(vars = response->variables; vars; vars = vars->next_variable)
	  print_variable(vars->name, vars->name_length, vars);
      
      vars = response->variables;   
      if (vars->type == ASN_IPADDRESS) {
	//u_char         *ip = vars->val.string;
	//printf("FOUND INDEX %d.%d.%d.%d\n", ip[0], ip[1], ip[2],ip[3]);	
	count++;
	bzero(buf,buflen);
	snprint_objid (buf,buflen,vars->name,vars->name_length);
	//printf("OID= %s\n",buf);
      }
    }
    if (response)
      snmp_free_pdu(response);
  }while (vars->type == ASN_IPADDRESS);

  ipRouteEntrys*=count;        /* GET THE NUMBER OF ENTRYS TO GET */
  pdu = snmp_pdu_create(SNMP_MSG_GETBULK);

  /* GET TABLE */
  pdu->non_repeaters = 0;
  pdu->max_repetitions = ipRouteEntrys;     
  sOID_len=MAX_OID_LEN;
  get_node(buf,sOID,&sOID_len);
  snmp_add_null_var(pdu, sOID, sOID_len);

  /*
   * Send the Request out.
   */
  status = snmp_synch_response(ss, pdu, &response);
  
  /*
   * Process the response.
   */
  if (status == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR) {
    /*
     * SUCCESS: Print the result variables
     */
    if (printresult)
      for(vars = response->variables; vars; vars = vars->next_variable)
	print_variable(vars->name, vars->name_length, vars);    
  } else {
    /*
     * FAILURE: print what went wrong!
     */
    if (status == STAT_SUCCESS)
      fprintf(stderr, "Error in packet\nReason: %s\n",
	      snmp_errstring(response->errstat));
    else
      snmp_sess_perror("snmpget", ss);
  }  
  if (response)
    snmp_free_pdu(response);

}

void help(){
  printf("usage: SNMPperftest [-hcnq?]\n\n");
  printf(" SNMPv2 based performance test of the Mib-2.interfaces table \n"); 
  printf("-h, --host HOSTNAME     address of the SNMP-agent        [DEFAULT= localhost]\n");
  printf("-c, --community STR     community to access the managementinformation  [DEFAULT= public]\n");
  //  printf("-n, --loops             number of calls to test time [DEFAULT= 100]\n");
  printf("-a, --average           number of loops to average time [DEFAULT= 1]\n");
  printf("-q  --quiet             supresses the output of the checked interfaces table\n");
  printf("-?, --help              this output\n");       

}



int main(int argc, char ** argv)
{
    struct snmp_session session, *ss;
    int loopnumber=100;
    int averagenumber=1;
    int printResult=1;
    struct timeval tvstart,tvend,tvdiff;
    struct timezone tzstart,tzend;
    double avgtime;
    double res1=0,res2=0;
    char hostname[255]="localhost";
    char community[255]="public";
    char optstring[12]="h:?n:a:c:q";
    int j;
    int i;
    int opt_idx;
    char c;
    static struct option long_options[] =
      {
	{"host", 1, 0, 'h'},
	{"number",1,0, 'n'},
	{"average",1,0, 'a'},
	{"quiet",1,0, 'q'},
	{"community",1,0, 'c'},
	{"help", 0, 0, '?'},
	{0, 0, 0, 0}
      };
    int count=0;
    int stepnum=9;
    int depth=9;
    int step[]={1, 10, 100, 250, 500, 1000, 2500, 5000, 10000};
  
  while((c=getopt_long(argc,argv,optstring,long_options,&opt_idx))!=-1){
    switch (c) {
    case 'n': depth=strtol(optarg,NULL,10);break;
    case 'a': averagenumber=strtol(optarg,NULL,10);break;
    case 'q': printResult=0;break;
    case 'h': strcpy(hostname,optarg);break;
    case 'c': strcpy(community,optarg);break; 
    case '?': help();exit(1);break;
    default: printf("UNKNOWN=%c",c);printf ("Option %s", long_options[opt_idx].name);
      break;
    }
  }

  printf("\n");
  if (optind < argc)
           {
             printf ("Nichtoptionselemente von ARGV: ");
             while (optind < argc)
             printf ("%s ", argv[optind++]);
             printf ("\n");
	     help();exit(1);
	   }

    /*
     * Initialize the SNMP library
     */
    init_snmp("snmpapp");

    /*
     * Initialize a "session" that defines who we're going to talk to
0     */
    snmp_sess_init( &session );                   /* set up defaults */
    session.peername = strdup(hostname);

    /* set the SNMP version number */
    session.version = SNMP_VERSION_2c;
 
    /* set the SNMPv1 community name used for authentication */
    session.community=community;
    session.community_len = strlen(session.community);
    

    /*
     * Open the session
     */
    SOCK_STARTUP;
    ss = snmp_open(&session);                     /* establish the session */

    if (!ss) {
        snmp_perror("ack");
        snmp_log(LOG_ERR, "something horrible happened!!!\n");
        exit(2);
    }
    

    printf("Starting Test...\n");
    printf("Number of loops to average result=%d\n",averagenumber);

    while((count<stepnum) && (count<depth)){
      loopnumber=step[count];
      avgtime=0;
      for(j=0;j<averagenumber;j++){
	gettimeofday(&tvstart,&tzstart);
	for(i=0;i<loopnumber;i++)  
	  getIF_snmpgetroute(ss,printResult);
	gettimeofday(&tvend,&tzend);
	
	
	tvdiff.tv_sec=tvend.tv_sec-tvstart.tv_sec;
	tvdiff.tv_usec=tvend.tv_usec-tvstart.tv_usec; 
	if (tvdiff.tv_usec<0){
	  tvdiff.tv_sec--;
	  tvdiff.tv_usec=1000000+tvdiff.tv_usec;
	}
	//printf("Lasted %ld,%06ld\n",tvdiff.tv_sec,tvdiff.tv_usec);
	avgtime+=tvdiff.tv_sec;
	avgtime+=(tvdiff.tv_usec*0.000001);
	if (j!=0) {
	  avgtime=avgtime/2.0;;
	}
      }
      res1=avgtime;

      //printf("****\nGETBULK needed an average time of %lfsec\n****\n",avgtime);
      avgtime=0;
      
      for(j=0;j<averagenumber;j++){
	gettimeofday(&tvstart,&tzstart);
	for(i=0;i<loopnumber;i++)  
	  getIF_snmpgetinterface(ss,printResult);
	gettimeofday(&tvend,&tzend);
	tvdiff.tv_sec=tvend.tv_sec-tvstart.tv_sec;
	tvdiff.tv_usec=tvend.tv_usec-tvstart.tv_usec; 
	if (tvdiff.tv_usec<0){
	  tvdiff.tv_sec--;
	  tvdiff.tv_usec=1000000+tvdiff.tv_usec;
	}
	//printf("Lasted %ld,%06ld\n",tvdiff.tv_sec,tvdiff.tv_usec);
	avgtime+=tvdiff.tv_sec;
	avgtime+=(tvdiff.tv_usec*0.000001);
	if (j!=0) {
	  avgtime=avgtime/2.0;;
	}
      } 
      res2=avgtime;
      
      printf("n=%6d  GETROUTE= %lf    GETINTERFACE= %lf  \n",loopnumber,res1,res2);
      //printf("****\nGET needed an average time of %lf\n****\n",avgtime);   
      count++;
    }
     printf("Test End...\n");
    //getIF_snmpget(ss,printResult);
    /*
     * Clean up:
     *  1) free the response.
     *  2) close the session.
     */
    snmp_close(ss);
    
    SOCK_CLEANUP;
    return (0);
} /* main() */
    
