// Webservice Device-Management 
// AGENT, eventmanagement.c
// Author: Oliver Bohle o.bohle@tu-bs.de
// Technische Universitt Braunschweig
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include "agent.h"


void print_regarr(){
  int i=0;
  struct in_addr in;

  printf("*****REGARR WITH %d ENTRYS**********\n",regarr_size);
  for (i=0;i<regarr_size;i++){
    in.s_addr=regarr[i].hostip;
    printf("%d: ID=%d %s:%d  flags=%d fail=%d\n",i,regarr[i].id,inet_ntoa(in),regarr[i].port,regarr[i].eventflags,regarr[i].fail);
  }
  printf("************************************\n");
}


unsigned long int getHostByteIP(char *hostname){
  struct hostent *hn;
  struct in_addr ip_adr,**addr_ptr;
  unsigned long int ip;
  if (inet_aton(hostname, &ip_adr)!=0)
    hn= gethostbyaddr((char *) &ip_adr,sizeof(struct in_addr),AF_INET);
  else 
    hn=gethostbyname(hostname);
  if (hn==NULL) return 0;
  addr_ptr=(struct in_addr **) hn->h_addr_list;
  ip=(**addr_ptr).s_addr;
  return ip;
}


/* translates the nsevt__regType flags in the internal flag representation */ 
int translate_regTypeFlags(enum nsevt__regType flag){
  switch (flag) {
  case EVT_REGTYPE_NONE:return EM_EVT_NONE;
  case EVT_REGTYPE_IF: return EM_EVT_INTERFACE;
  case EVT_REGTYPE_RT: return EM_EVT_ROUTE;
  case EVT_REGTYPE_ALL:return EM_EVT_ALL;
  default: return -1;
  }

  return -1;

}

/* checks, if the given flag is a valid flag for registration */
int validateFlagsForRegistration(enum nsevt__regType flag){
  switch (flag) {
  case EVT_REGTYPE_NONE: return 1;
  case EVT_REGTYPE_IF: return 1;
  case EVT_REGTYPE_RT: return 1;
  case EVT_REGTYPE_ALL:return 1;
  default: return 0;
  }
  return 0;
}



struct notify * deleteRegEntry(struct notify *nt,int *ntsize,int ctr){
  struct notify *tmp;
  int tmpctr=0,i=0;

  if (*ntsize==1){
    *ntsize=0;
    free(nt);
    nt=NULL;
    return nt;
  }else{
    tmp=nt;
    *ntsize=*ntsize-1;
    nt=(struct notify *) calloc(*ntsize,sizeof(struct notify));
    if (nt==NULL) error("unregister_eventManager","Not enough Memory",-2); 
    tmpctr=0;
    for(i=0;i<*ntsize+1;i++)
      if (i!=ctr) {
	printf("FILLING REGARR FIELD tmpctr=%d i=%d\n",tmpctr,i);
	nt[tmpctr].id=tmp[i].id;
	nt[tmpctr].hostip=tmp[i].hostip;
	nt[tmpctr].port=tmp[i].port;
	nt[tmpctr].eventflags=tmp[i].eventflags;
	nt[tmpctr].fail=tmp[i].fail;
	tmpctr++;
      }
    free(tmp);
  }
  return nt;
}







int register_eventManager(int *id,char* hostname,int port,enum nsevt__regType flag){
  unsigned long int  ip;
  int ctr=0;
 
  if (!validateFlagsForRegistration(flag)) return EM_WRONGREGTYPE;
 
  if (*id>0){ 
    while ((ctr<regarr_size) && (regarr[ctr].id!=*id)) ctr ++;
    if (!(ctr<regarr_size)) return EM_WRONGID;
    ip=getHostByteIP(hostname);
    if ((ctr<regarr_size) && ((regarr[ctr].hostip!=ip) || (regarr[ctr].port!=port))) return EM_WRONGID;
  }else{
    ip=getHostByteIP(hostname);
    printf("REGISTER %s:%d Flag=%d\n",hostname,port,flag);
    if (ip!=0)
      while((ctr<regarr_size) && ((regarr[ctr].hostip!=ip) || (regarr[ctr].port!=port))) ctr++;
  }

  if (!(ctr<regarr_size)) {             /* new Entry */
    regarr=(struct notify *) realloc(regarr,++regarr_size*sizeof(struct notify));
    if (regarr==NULL) error("register_eventManager","Not enough Memory",-2);
    regarr[regarr_size-1].id=lastid++;
    regarr[regarr_size-1].hostip=ip;
    regarr[regarr_size-1].port=port;
    regarr[regarr_size-1].eventflags=translate_regTypeFlags(flag);
    regarr[regarr_size-1].fail=0;
    *id=regarr[regarr_size-1].id;
#ifdef AGENT_DEBUG
    print_regarr();
#endif
    return EM_SUCCESS;
  }else{
    printf("CHANGING FLAGS\n");
    regarr[ctr].eventflags=translate_regTypeFlags(flag);
    return EM_SUCCESS;
  }

  return EM_WRONGHOSTNAME;
}


int unregister_eventManager(int *id,char* hostname,int port,enum nsevt__regType flag){
  int ctr=0;
  unsigned long int  ip;


  if (!validateFlagsForRegistration(flag)) return EM_WRONGREGTYPE;

  ip=getHostByteIP(hostname);
  printf("UNREGISTER ID=%d %s:%d Flag=%d\n",*id,hostname,port,flag);
 
  /* Check for valid id */
  if (*id>0){ 
    while ((ctr<regarr_size) && (regarr[ctr].id!=*id)) ctr ++;
    if (!(ctr<regarr_size)) return EM_WRONGID;
    ip=getHostByteIP(hostname);
    if ((ctr<regarr_size) && ((regarr[ctr].hostip!=ip) || (regarr[ctr].port!=port))) return EM_WRONGID;
  }else return EM_WRONGID;
 
  if (flag==EVT_REGTYPE_ALL){ /* delete the Entry */
    regarr=deleteRegEntry(regarr,&regarr_size,ctr);

#ifdef AGENT_DEBUG
    print_regarr();
#endif

    return EM_SUCCESS;
  }else{
    int tmpflag;
    printf("CHANGING FLAGS\n");
    tmpflag=translate_regTypeFlags(flag); 
    if (tmpflag & regarr[ctr].eventflags) regarr[ctr].eventflags-=tmpflag;
    else return EM_FLAGNOTSET;

#ifdef AGENT_DEBUG
    print_regarr();
#endif
    return EM_SUCCESS;
  }
  return EM_WRONGHOSTNAME;
}


int intrestedInMessage(enum nsevt__type msgtype,int flags){
  
  if (flags & EM_EVT_ALL) return 1;
  switch (msgtype) {
  case EVT_TP_IF:if (flags & EM_EVT_INTERFACE) return 1;break;
  case EVT_TP_ROUTE:if (flags & EM_EVT_ROUTE) return 1;break;
  case EVT_TP_DEFAULT:if (flags & EM_EVT_ALL) return 1;break;
  case EVT_TP_SRVCLOSE:return 1;break; 
  default: return 0;  
  }
  return 0;
}




void sendEventMessage(char *msg,enum nsevt__type  type){
  int i=0;
  int fault=0;
  char option[60];
  char *sndmsg;
  struct in_addr in;
  enum nsevt__type rt;    /* return type */ 

 


#ifdef AGENT_DEBUG 
  printf("EVENT OCCURED: %s\n",msg);
  print_regarr();
#endif

  sndmsg=(char *) soap_malloc(rm_soap,strlen(msg)*sizeof(char));
  strcpy(sndmsg,msg);
  for (i=0;i<regarr_size;i++){

    if (intrestedInMessage(type,regarr[i].eventflags)){
      if (!rm_soap->header) {
	rm_soap->header = soap_malloc(rm_soap,sizeof(struct SOAP_ENV__Header)); // alloc new header
	rm_soap->header->connection_ID = regarr[i].id; 
      }else{
	rm_soap->header->connection_ID = regarr[i].id; 
      }     
      bzero(option,sizeof(option));
      in.s_addr=regarr[i].hostip;
      sprintf(option,"%s:%d",inet_ntoa(in),regarr[i].port);
#ifdef AGENT_DEBUG
      printf("CALLING %s SOAP=%p\n",option,rm_soap);
#endif
      fault=soap_call_nsevt__signalEvent(rm_soap,option,"",sndmsg,type,&rt);
#ifdef AGENT_DEBUG
      printf("Errorcode: %d\n",fault);
#endif
      if (fault!=SOAP_OK) {
	regarr[i].fail++;
	soap_print_fault(rm_soap, stderr);
      }else
	regarr[i].fail=0;
    }
  }
  /* ** FAILURE MANAGEMENT ** */
  for (i=0;i<regarr_size;i++)
    if (regarr[i].fail >= EM_MAX_TRIES){
      int id=regarr[i].id;
      in.s_addr=regarr[i].hostip;
      unregister_eventManager(&id,inet_ntoa(in),regarr[i].port,EVT_REGTYPE_ALL);
    }
}
