// Webservice Device-Management 
// AGENT, interfaces.c
// Author: Oliver Bohle o.bohle@tu-bs.de
// Technische Universitt Braunschweig

#include "agent.h"
#include "../headers/SOAPdevmgrStub.h"

int *match;
int match_size;




/* prints a interfaceDescription struct for debugging purposes */
void print_interfaceDescription(struct nsif__interfaceDescription *ifd){
  int i;
  printf("*************INTERFACE DESCRIPTION****************\n");
  printf("IFNAME          : %s\n",ifd->name);
  printf("MTU             : %d\n",ifd->mtu);
  printf("INET-ADDR       : ");
  if (ifd->address==NULL) printf("NULL");
  else print_ns__address(ifd->address);
  printf("\n");
  printf("NETMASK         : ");
  if (ifd->netmask==NULL) printf("NULL");
  else print_ns__address(ifd->netmask);
  printf("\n");
  printf("BROADCAST       : ");
  if (ifd->broadcast==NULL) printf("NULL");
  else print_ns__address(ifd->broadcast);
  printf("\n");
  printf("DST-ADDR        : ");
  if (ifd->dst_address==NULL) printf("NULL");
  else print_ns__address(ifd->dst_address);
  printf("\n");
  printf("LINK-ADDR       : ");
  if (ifd->link_address==NULL) printf("NULL");
  else print_ns__address(ifd->link_address);
  printf("\n");
  printf("ALIAS           : ");
  for(i=0;i<ifd->__sizealias;i++){
    print_ns__address(ifd->alias[i]);
    printf(" ");
  }
  printf("\n");
  printf("FLAGS           : ");
  for(i=0;i<ifd->__sizeflags;i++){
    if(i>0) printf(", ");
    printf("%s",ifd->flag[i]);
    
  }
  printf("\n");
  printf("***********************************************\n");
	 
}



/* parses the parameter flags for DNET flag-types and stores them as strings
   in the flagContainer struct iffc */

struct flagContainer * getInterfaceFlags(u_short flags,struct flagContainer *iffc) { 

  int ctr=0; 
  char **strarr=NULL;
  
  if (flags & INTF_FLAG_UP){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "UP", AGENT_FLAGLEN);
  }
  if (flags & INTF_FLAG_LOOPBACK){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "LOOPBACK", AGENT_FLAGLEN);
  }         
  if (flags & INTF_FLAG_POINTOPOINT){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "POINTOPOINT", AGENT_FLAGLEN);
  }
  if (flags & INTF_FLAG_NOARP) {
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "NOARP", AGENT_FLAGLEN);
  }
  if (flags & INTF_FLAG_BROADCAST){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "BROADCAST", AGENT_FLAGLEN);
  }
  if (flags & INTF_FLAG_MULTICAST){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "MULTICAST", AGENT_FLAGLEN);
  }

  iffc->array=strarr;
  iffc->size=ctr;

  return iffc;
}

/* Returns the integer representation of the flags in id */
int setInterfaceFlags(struct nsif__interfaceDescription *id){
  int i,flagval=0;

  for(i=0;i<id->__sizeflags;i++){
    if (strcmp(id->flag[i],"UP")==0) {flagval+= INTF_FLAG_UP;continue;}
    if (strcmp(id->flag[i],"LOOPBACK")==0) {flagval+= INTF_FLAG_LOOPBACK;continue;}
    if (strcmp(id->flag[i],"POINTOPOINT")==0) {flagval+= INTF_FLAG_POINTOPOINT;continue;}
    if (strcmp(id->flag[i],"NOARP")==0) {flagval+= INTF_FLAG_NOARP;continue;}
    if (strcmp(id->flag[i],"BROADCAST")==0) {flagval+= INTF_FLAG_BROADCAST;continue;}
    if (strcmp(id->flag[i],"MULTICAST")==0) {flagval+= INTF_FLAG_MULTICAST;continue;} 
  }
  return flagval;
  
}


/* checks if the Flags in newflags are equal to the flags in id. If
   true, the return value is 1 else 0. rfc shall be a
   routeFlagsContainer array with two entrys. in the first one the
   deleted flags are set, in the second one the new ones. */

int equalIFFlags(int newflags,struct nsif__interfaceDescription *id,struct flagContainer *iffc){
  
   int oldflags;
   int new=0, del=0;  
   /* comparing to find differnt flags */
   oldflags=setInterfaceFlags(id);

   if (newflags==oldflags) return 1;

   /* FLAGS are different */
   
   checkFlagChange(oldflags,newflags,INTF_FLAG_UP,&new,&del);
   checkFlagChange(oldflags,newflags,INTF_FLAG_LOOPBACK,&new,&del);
   checkFlagChange(oldflags,newflags,INTF_FLAG_POINTOPOINT,&new,&del);
   checkFlagChange(oldflags,newflags,INTF_FLAG_NOARP,&new,&del);
   checkFlagChange(oldflags,newflags,INTF_FLAG_BROADCAST,&new,&del);
   checkFlagChange(oldflags,newflags,INTF_FLAG_MULTICAST,&new,&del);
   
   
   if (del!=0) getInterfaceFlags (del,&iffc[0]);
   if (new!=0) getInterfaceFlags (new,&iffc[1]);

   return 0;
}


/* Translates a DNET adress structure into a ns__address structure */
struct ns__address *  getAddress(struct ns__address *outaddr,struct addr adr){

  outaddr=(struct ns__address *) malloc(sizeof(struct ns__address));
  if (adr.addr_type==ADDR_TYPE_NONE) {outaddr=NULL,free(outaddr);return NULL;}
  if (adr.addr_type==ADDR_TYPE_IP){
    outaddr->type=IP;
    outaddr->addr=(char *) calloc(AGENT_IPLEN,sizeof(char));
    strncat(outaddr->addr,ip_ntoa(&adr.addr_ip) ,AGENT_IPLEN);
    return outaddr;
  }
  if (adr.addr_type==ADDR_TYPE_ETH){
    outaddr->type=ETH;
    outaddr->addr=(char *) calloc(AGENT_ETHLEN,sizeof(char));
    strncat(outaddr->addr,addr_ntoa(&adr) ,AGENT_ETHLEN);
    return outaddr;
  }
  warning("interaces.c:getAddress","new interface type found");
  return outaddr;

}



/* Translates the DNET interface types into nsif__type types */ 
int getIFType(int intf_type){
  /* *** TYPE *** */
  int tmp=0;
  switch (intf_type) {
  case INTF_TYPE_OTHER: tmp=IF_OTHER;break;
  case INTF_TYPE_ETH: tmp=IF_ETHERNET;break;
  case INTF_TYPE_LOOPBACK: tmp=IF_LOOPBACK;break;
  case INTF_TYPE_TUN: tmp=IF_VIRTUAL;break;
  default:tmp=IF_OTHER;break;
  }
  return tmp;
}



#define BUFLEN 100


/* Frees the memory allocation of the given nsif__interfaceDescription
   struct */
void freeInterfaceDescription(struct nsif__interfaceDescription *id){
  int j;
  
  freeAddrStruct(id->address);
  freeAddrStruct(id->dst_address);
  freeAddrStruct(id->link_address); 
  free(id->name);
  for (j=0;j<id->__sizeflags;j++) free(id->flag[j]);
  free(id->flag);
 
  for (j=0;j<id->__sizealias;j++) {
    freeAddrStruct(id->alias[j]);
  }
  free(id->alias);
}




/* will be called by intf_loop called in create_InterfaceList(). It
   fills a nsif__interfaceDescription struct with the information from
   entry. The global ifDescr struct will be expanded with this new
   entry*/
int create_interface_entry(const struct intf_entry *entry, void *arg)
{
  struct addr bcast;
  uint32_t mask;
  int i;
  struct flagContainer iffc;
  struct ns__address *nmaddr;


  /* Memory allocation */
  if (ifDescr==NULL){
    ++ifDescr_size;
    ifDescr=(struct nsif__interfaceDescription *) calloc(1,sizeof(struct nsif__interfaceDescription));
  }else{
    ++ifDescr_size;
    ifDescr=(struct nsif__interfaceDescription *) realloc(ifDescr,ifDescr_size*sizeof(struct nsif__interfaceDescription));
  }
  if (ifDescr==NULL) error("create_interface_list","Not enough Memory",-2);
  memset(&ifDescr[ifDescr_size-1],0,sizeof(struct nsif__interfaceDescription));

  /* Filling structure with interface stats */

  /* *** NAME *** */
  ifDescr[ifDescr_size-1].name=(char *) calloc(AGENT_IFNAMELEN,sizeof(char));
  strncat(ifDescr[ifDescr_size-1].name,entry->intf_name , AGENT_IFNAMELEN);
  

  /* *** TYPE *** */
  switch (entry->intf_type) {
  case INTF_TYPE_OTHER: ifDescr[ifDescr_size-1].type=IF_OTHER;break;
  case INTF_TYPE_ETH: ifDescr[ifDescr_size-1].type=IF_ETHERNET;break;
  case INTF_TYPE_LOOPBACK: ifDescr[ifDescr_size-1].type=IF_LOOPBACK;break;
  case INTF_TYPE_TUN: ifDescr[ifDescr_size-1].type=IF_VIRTUAL;break;
  default:ifDescr[ifDescr_size-1].type=IF_OTHER;break;
  }
  /* *** MTU *** */
  ifDescr[ifDescr_size-1].mtu=entry->intf_mtu;
  
  
  /* *** FLAGS *** */
  getInterfaceFlags(entry->intf_flags,&iffc);
  ifDescr[ifDescr_size-1].flag=iffc.array;
  ifDescr[ifDescr_size-1].__sizeflags=iffc.size;

  /* *** ADDRESS *** */
  
  ifDescr[ifDescr_size-1].address=getAddress(ifDescr[ifDescr_size-1].address,entry->intf_addr);
  ifDescr[ifDescr_size-1].dst_address=getAddress(ifDescr[ifDescr_size-1].dst_address,entry->intf_dst_addr);
  ifDescr[ifDescr_size-1].link_address=getAddress(ifDescr[ifDescr_size-1].link_address,entry->intf_link_addr);

  /* BROADCAST, NETMASK */

  if ((ifDescr[ifDescr_size-1].address!=NULL)&&(ifDescr[ifDescr_size-1].address->type==IP)){ 

    addr_btom(entry->intf_addr.addr_bits, &mask, IP_ADDR_LEN);
    addr_bcast(&entry->intf_addr, &bcast);

    ifDescr[ifDescr_size-1].broadcast=getAddress(ifDescr[ifDescr_size-1].broadcast,bcast);
    nmaddr=(struct ns__address *) malloc(sizeof(struct ns__address));
    nmaddr->type=IP;
    nmaddr->addr=(char *) calloc(AGENT_IPLEN,sizeof(char));
    strncat(nmaddr->addr,ip_ntoa((ip_addr_t *) &mask) ,AGENT_IPLEN);
    ifDescr[ifDescr_size-1].netmask=nmaddr;
    
  }else{
    ifDescr[ifDescr_size-1].netmask=NULL;
    ifDescr[ifDescr_size-1].broadcast=NULL;
  }
    
  /* *** ALIASLIST *** */


  if (entry->intf_alias_num==0) {
    ifDescr[ifDescr_size-1].__sizealias=0;
    ifDescr[ifDescr_size-1].alias=NULL;
  }else{
    ifDescr[ifDescr_size-1].__sizealias=entry->intf_alias_num;
    ifDescr[ifDescr_size-1].alias=(struct ns__address **) calloc(entry->intf_alias_num,sizeof(struct ns__address *));
  for (i = 0; i < entry->intf_alias_num; i++)
    ifDescr[ifDescr_size-1].alias[i]=getAddress(ifDescr[ifDescr_size-1].alias[i],entry->intf_alias_addrs[i]);
  }
  
  return (0);
}



/* this function shall be called first to fetch interface information
   and fill the ifDescr struct */
void create_InterfaceList(){ 
  intf_t *it=NULL; 
  int result=0;
 
  if (ifDescr!=NULL)
    warning("createInterfaceList","ifDescr pointer not NULL::possible memory leak");
   
  ifDescr=NULL;
  ifDescr_size=0;

  it=intf_open(); 
  result=intf_loop(it,create_interface_entry,NULL);
  intf_close(it);
  
}


/* checks if the given addresses are equal. Returns 0 if they are equal, else 1; */

int checkIFaddress(struct ns__address *orig,struct ns__address *addr){
  int diff=0;
  if (orig!=NULL){
    if (addr!=NULL) {
      if ((addr->type!=orig->type) || (strcmp(addr->addr,orig->addr)!=0)){
	diff=1;
      }
    }else{
      diff=1;
    }
  }else{
    if (addr!=NULL) {
      diff=1;
    }
  }
  return diff;
}


/* checks if the two alias lists in oldalias and entry are the same,
   if not the diffrences wil be stored in the deladdr and newaddr
   arrays. The reference for this array is oldalias. The returnvalue
   ist 1 if a difference is found, else 0 
*/
int checkAliasList(struct ns__address **oldalias,int oldalias_size,const struct intf_entry *entry,struct ns__address **delad,int *deladdr_size,struct ns__address **newad,int *newaddr_size){
  int oldmatch[oldalias_size],newmatch[entry->intf_alias_num];
  struct ns__address *deladdr,*newaddr;
  int i,j;
  struct ns__address *addr;
  int diff=0;

  for (i=0;i<oldalias_size;i++) oldmatch[i]=0;
  for (i=0;i<entry->intf_alias_num;i++) newmatch[i]=0;

  
  for (i=0;i<oldalias_size;i++){
    j=0;
    addr=getAddress(addr,entry->intf_alias_addrs[j]);
    while((checkIFaddress(oldalias[i],addr)==1) && (j<entry->intf_alias_num)) {
      j++;
      freeAddrStruct(addr);
      if (j<entry->intf_alias_num) addr=getAddress(addr,entry->intf_alias_addrs[j]);
    }
  
    if (j<entry->intf_alias_num){
      /* FOUND */
      oldmatch[i]=1;
      newmatch[j]=1;
    }
  }

  /* check if a difference was found and subscribe it in deladdr and newaddr */
  *deladdr_size=0;
  deladdr=NULL;
  for (i=0;i<oldalias_size;i++)
    if (oldmatch[i]==0) {
      diff=1;
      deladdr=(struct ns__address *) realloc(deladdr,++(*deladdr_size) * sizeof(struct ns__address));
      deladdr[*deladdr_size-1].type=oldalias[i]->type;
      if (oldalias[i]->type==ETH){
	deladdr[*deladdr_size-1].addr=(char *) calloc(AGENT_ETHLEN,sizeof(char));
	strncpy(deladdr[*deladdr_size-1].addr,oldalias[i]->addr ,AGENT_ETHLEN);
      }
      if (oldalias[i]->type==IP){
	deladdr[*deladdr_size-1].addr=(char *) calloc(AGENT_IPLEN,sizeof(char));
	strncpy(deladdr[*deladdr_size-1].addr,oldalias[i]->addr ,AGENT_IPLEN);
      }
    };
  *newaddr_size=0;
  newaddr=NULL;
  for (i=0;i<entry->intf_alias_num;i++)
    if (newmatch[i]==0) {
      struct ns__address *addr=NULL;
      diff=1;     
      newaddr=(struct ns__address *) realloc(newaddr,++(*newaddr_size) * sizeof(struct ns__address));
      if (newaddr==NULL) error("interfaces.c:checkAliasList",":MEMORY allocation error",-2);
      addr=getAddress(addr,entry->intf_alias_addrs[i]);
      if (addr!=NULL){
	if (addr->addr!=NULL){
	  newaddr[*newaddr_size-1].addr=addr->addr;
	  addr->addr=NULL;
	}
	newaddr[*newaddr_size-1].type=addr->type;
      }
      freeAddrStruct(addr);
    }
  *delad=deladdr;
  *newad=newaddr;
  return diff;
}







/* checks if the interface description given by the two parameters are
   the same. if they are equal, the returnvalue is 0 else 1. The
   equality of the interface names and types will not be checked. If a
   difference is found the differences will be updated in the global
   ifDescr struct and a eventmessage will be sent. (called by
   update_interface_entry()) */ 
int check_interface_information(const struct intf_entry *entry,struct nsif__interfaceDescription *id){
  int diff=0;
  struct ns__address *addr,*deladdr,*newaddr;
  struct flagContainer rfc[2],updrfc;
  int deladdr_size=0,newaddr_size=0;
  int i;
  struct addr bcast;
  uint32_t mask;

  char msg[AGENT_ERRMSGLEN];
  int ret;

  rfc[0].size=0;
  rfc[0].array=NULL;
  rfc[1].size=0;
  rfc[1].array=NULL;

	   
  ret=EVT_TP_IF;
  bzero(msg,sizeof(msg));
	    

  sprintf(msg,"UPDATE: INTERFACE :: IFNAME=%s",id->name);
	   	    

  /* *** MTU *** */
  if (id->mtu!=entry->intf_mtu) {diff=1;sprintf(msg,"%s MTU %d->%d",msg,id->mtu,entry->intf_mtu);id->mtu=entry->intf_mtu;}
  
  /* *** ADDRESS *** */
  addr=getAddress(addr,entry->intf_addr);
  if (checkIFaddress(id->address,addr)) { 
    diff=1;
    if (id->address==NULL)
      sprintf(msg,"%s ADDRESS NONE->%s",msg,addr->addr);
    else
      if (addr==NULL)
	sprintf(msg,"%s ADDRESS %s->NONE",msg,id->address->addr);
      else
	sprintf(msg,"%s ADDRESS %s->%s",msg,id->address->addr,addr->addr);
    freeAddrStruct(id->address);  
    id->address=addr;
    addr=NULL;
  }else
    freeAddrStruct(addr);

  addr=getAddress(addr,entry->intf_dst_addr);
  if (checkIFaddress(id->dst_address,addr)){     
    diff=1;
    if (id->dst_address==NULL)
      sprintf(msg,"%s DST_ADDRESS NONE->%s",msg,addr->addr);
    else
      if (addr==NULL)
	sprintf(msg,"%s DST_ADDRESS %s->NONE",msg,id->dst_address->addr);
      else
	sprintf(msg,"%s DST_ADDRESS %s->%s",msg,id->dst_address->addr,addr->addr);
    freeAddrStruct(id->dst_address);
    id->dst_address=addr;
    addr=NULL;
   
  }else
    freeAddrStruct(addr);

  addr=getAddress(addr,entry->intf_link_addr);
  if (checkIFaddress(id->link_address,addr)){   
    diff=1;
    if (id->link_address==NULL)
      sprintf(msg,"%s LINK_ADDRESS NONE->%s",msg,addr->addr);
    else
      if (addr==NULL)
	sprintf(msg,"%s LINK_ADDRESS %s->NONE",msg,id->link_address->addr);
      else
	sprintf(msg,"%s LINK_ADDRESS %s->%s",msg,id->link_address->addr,addr->addr);
    freeAddrStruct(id->link_address);
    id->link_address=addr;
    addr=NULL;
  }else   
  freeAddrStruct(addr);


  
  if (id->address==NULL){
    id->broadcast=NULL;
    id->netmask=NULL;
  }else{
    /* BROADCAST, NETMASK */
    addr_btom(entry->intf_addr.addr_bits, &mask, IP_ADDR_LEN);
    addr_bcast(&entry->intf_addr, &bcast); 
    addr=getAddress(addr,bcast);  
  
    if (checkIFaddress(id->broadcast,addr)){
      if (id->broadcast==NULL)
	sprintf(msg,"%s BROADCAST_ADDESS NONE->%s",msg,addr->addr);
      else
	if (addr==NULL)
	  sprintf(msg,"%s BROADCAST_ADDRESS %s->NONE",msg,id->broadcast->addr);
	else
	  sprintf(msg,"%s BROADCAST_ADDRESS %s->%s",msg,id->broadcast->addr,addr->addr);   
      freeAddrStruct(id->broadcast);
      id->broadcast=addr;
      addr=NULL;
      diff=1;
    }  
    
    freeAddrStruct(addr);

    /* NETMASK CHECK */
    addr=(struct ns__address *) malloc(sizeof(struct ns__address));
    addr->type=IP;
    addr->addr=(char *) calloc(AGENT_IPLEN,sizeof(char));
    strncat(addr->addr,ip_ntoa((ip_addr_t *) &mask) ,AGENT_IPLEN);
    
    if (checkIFaddress(id->netmask,addr)){
      if (id->netmask==NULL)
	sprintf(msg,"%s NETMASK_ADDESS NONE->%s",msg,addr->addr);
      else
	if (addr==NULL)
	  sprintf(msg,"%s NETMASK_ADDRESS %s->NONE",msg,id->netmask->addr);
	else
	  sprintf(msg,"%s NETMASK_ADDRESS %s->%s",msg,id->netmask->addr,addr->addr); 
      freeAddrStruct(id->netmask);
      id->netmask=addr;
      addr=NULL;
      diff=1;
    }  
    freeAddrStruct(addr);
  }

  /* *** FLAGS *** */
  //printf("FLAGS=%d!=%d ?\n",setInterfaceFlags(id),entry->intf_flags);

  if (!(equalIFFlags(entry->intf_flags,id,rfc))) {
    diff=1;
    printf("FLAGS CHANGED\n");
    getInterfaceFlags(entry->intf_flags,&updrfc);
    free(id->flag);
    id->flag=updrfc.array;
    id->__sizeflags=updrfc.size;
  }

  /* *** manage reporting of changed flags *** */
  if (rfc[0].size>0){
    int i=0;
    sprintf(msg,"%s DELETED FLAGS = <",msg);
    for (i=0;i<rfc[0].size;i++) sprintf(msg,"%s %s",msg,rfc[0].array[i]);
    sprintf(msg,"%s >",msg);
  }
  if (rfc[1].size>0){
    int i=0;
    sprintf(msg,"%s ADDED FLAGS = <",msg);
    for (i=0;i<rfc[1].size;i++) sprintf(msg,"%s %s",msg,rfc[1].array[i]);
    sprintf(msg,"%s >",msg);
  }



  /* *** ALIAS *** */
  deladdr_size=0;newaddr_size=0;
  deladdr=NULL;newaddr=NULL;
  if (checkAliasList(id->alias,id->__sizealias,entry,&deladdr,&deladdr_size,&newaddr,&newaddr_size)) {
    diff=1;
    
    for(i=0;i<id->__sizealias;i++){
      freeAddrStruct(id->alias[i]);
    }

    
    /* Building new alias list to update entry in ifDescr list */
    if (entry->intf_alias_num==0) {
      id->__sizealias=0;
      id->alias=NULL;
    }else{
      id->__sizealias=entry->intf_alias_num;
      id->alias=(struct ns__address **) calloc(entry->intf_alias_num,sizeof(struct ns__address *));
      for (i = 0; i < entry->intf_alias_num; i++)
	id->alias[i]=getAddress(id->alias[i],entry->intf_alias_addrs[i]);
    }  
  }
  
  
  /* *** manage reporting of changed alias *** */
  if (deladdr_size>0){
    int i=0;
    sprintf(msg,"%s DELETED ALIAS = <",msg);
    for (i=0;i<deladdr_size;i++) sprintf(msg,"%s %s",msg,deladdr[i].addr);
    sprintf(msg,"%s >",msg);
  }
  if (newaddr_size>0){
    int i=0;
    sprintf(msg,"%s ADDED ALIAS = <",msg);
    for (i=0;i<newaddr_size;i++) sprintf(msg,"%s %s",msg,newaddr[i].addr);
    sprintf(msg,"%s >",msg);
  }


  if (diff) sendEventMessage(msg,ret );
#ifdef AGENT_DEBUG  
  if (diff) {
    printf("************* CHANGED INTERFACE-ENTRY ****************\n");
    print_interfaceDescription(id);
  }
#endif
  return diff;
}


/* checks, if a given interface description can be found in the
   previously build global ifDescr. If an entry is found, the
   corresponding entry in the match-array, which has to be allocated
   before, will be set to 1 and a paramtercheck is invoked by
   check_interface_information. If a new interface-description is
   found, it will be appended to the ifDescr array and a eventmessage
   will be sent. This function is called by
   update_InterfaceList().  */
int update_interface_entry(const struct intf_entry *entry, void *arg)
{
  int i;
  char msg[AGENT_ERRMSGLEN];
  int ret;

  for(i=0;i<match_size;i++)
    
    if (strcmp(entry->intf_name,ifDescr[i].name)==0) {
      match[i]=1;
      /* check for changed interface-parameters */
      check_interface_information(entry,&ifDescr[i]);
      return (0);
    }

  /* appending the new entry at the end of the ifDescr */
  create_interface_entry(entry,NULL);

  /* sending this event to interested receivers */ 
  bzero(msg,sizeof(msg));
  sprintf(msg,"NEW INTERFACE :: IFNAME=%s",entry->intf_name);
  ret=EVT_TP_IF;
  sendEventMessage(msg,ret );
  
  return (0);
}



/* updates the ifDescr struct. First a global "match" field is
   created. for each ifDescr entry exists one entry in this field. The
   function update_interface_entry is indirectly called by this
   function and if an interface entry is found in the ifDescr struct
   the appropriate match entry will be marked. If there are not marked
   fields at the end of intf_loop these entrys are not longer actual
   and they will be deleted from the ifDescr struct. For each deleted
   entry an eventmessage will be sent */
void update_InterfaceList(){
  intf_t *it=NULL;
  int result=0,i=0,todelete=0,tmp_size=0,count=0;
  struct nsif__interfaceDescription *idtmp;
  
#ifdef AGENT_DEBUG 
  printf("UPDATE_INTERFACE_LIST\n");
#endif
 

  it=intf_open();

  if  (ifDescr==NULL) {
    create_InterfaceList();
    return;
  }else{
    match=(int *) calloc(ifDescr_size,sizeof(int));
    match_size=ifDescr_size;
    result=intf_loop(it,update_interface_entry,NULL);

    /* remove the not marked entrys in the match array */

    todelete=0;
    /* check for deleted entrys */ 
    for (i=0;i<match_size;i++) {
      if (!match[i]) todelete++;
    }
    if (todelete){
      /* delete entrys */
  
      idtmp=ifDescr;
      tmp_size=ifDescr_size;
      ifDescr=(struct nsif__interfaceDescription *) calloc(ifDescr_size-todelete,sizeof(struct nsif__interfaceDescription));
      if (ifDescr==NULL) error("updateInterfaceList","Not enough Memory",-2);
      ifDescr_size=ifDescr_size-todelete;
      count=0;
            
      for (i=0;i<tmp_size;i++) {
	if (i<match_size) {
	  if (match[i]) {
	    ifDescr[count++]=idtmp[i];
	  }else{
	    char msg[AGENT_ERRMSGLEN];
	    int ret;
	    /* DELETED ENTRY FOUND */	   
	    bzero(msg,sizeof(msg));
	    sprintf(msg,"DELETED INTERFACE :: IFNAME=%s",idtmp[i].name);
	    ret=EVT_TP_IF;
	    sendEventMessage(msg,ret );
	    freeInterfaceDescription(&idtmp[i]);
	  }
	}else{
	  ifDescr[count++]=idtmp[i];
	}	
      }
      /* free idtmp */
      free(idtmp);
    }

  }
  
  intf_close(it);

}

/* extracts the received information in "in" to a libdnet intf_entry
   struct. With this struct the interfaceinformation will be set */
int setInterface(struct nsif__interfaceDescription in,char *errmsg){
  intf_t *it=NULL;
  struct intf_entry *entry;
  int error=0;
  struct addr *ap;
  char buf[1024];
  
#ifdef AGENT_DEBUG
  printf("*'*'*'*'*'*'*'* INTERFACE INFORMATION TO SET *'*'*'*'*'*'*'*'*'\n");
  print_interfaceDescription(&in);
#endif

  
  /** intialisation like in intf.c of the dnet test-program **/
  entry=(struct intf_entry *) buf;
  memset(entry,0,sizeof(*entry));
  entry->intf_len=(sizeof(buf));
  
  if ((in.name==NULL) || (strcmp(in.name,"")==0)){
    sprintf(errmsg,"No interface name given in SET interface request");
    return -1;
  }
  strncpy(entry->intf_name,in.name,sizeof(entry->intf_name));
  
  if (in.address!=NULL){
    ap=&entry->intf_addr;
    if (addr_pton(in.address->addr,ap)<0){
      sprintf(errmsg,"inet addr: %s",strerror(errno));
      return -1;
    }
  }
  if (in.dst_address!=NULL){
    ap=&entry->intf_dst_addr;
    if (addr_pton(in.dst_address->addr,ap)<0){
      sprintf(errmsg,"inet addr: %s",strerror(errno));
      return -1;
    }
  }
  if (in.link_address!=NULL){
    ap=&entry->intf_link_addr;
    if (addr_pton(in.link_address->addr,ap)<0){
      sprintf(errmsg,"inet addr: %s",strerror(errno));
      return -1;
    }
  }
  if (in.__sizeflags>0){
    entry->intf_flags=setInterfaceFlags(&in);
  }


  if (in.__sizealias>0){
    int i=0;
    for (i=0;i<in.__sizealias;i++){
      ap=&entry->intf_alias_addrs[entry->intf_alias_num++];
      if (addr_pton(in.alias[i]->addr,ap)<0){
	sprintf(errmsg,"inet addr: %s",strerror(errno));
	return -1;
      }
    }
  }

  it=intf_open();
  error=intf_set(it,entry);
  intf_close(it);
  if (error<0) {
    sprintf(errmsg,"%s",strerror(errno));
    return -1;
  }
  return 1;
}










/* function do deallocate old ifStat arrays */
void deallocOldIfStats(){
  int i;
  for (i=0;i<ifStat_size;i++){
    free(ifStat[i].ifname);
  }
  free(ifStat);
  ifStat=NULL;
  ifStat_size=0;

}



/* this function fills the ifStat array with the actual status
   information of the interfaces. If a ifname is set, the array is
   only filled with the interfaces status information of the interface
   which name is ifname. If no interfacename matches ifname, ifStats
   will be an empty array. The status information are fetched from
   /proc/net/dev */
void getIfStats(char *ifname){
  FILE *fd;
  char buf[BUFLEN]="";
  char *bufptr=buf;
  char iface[16]="";
  char tmp[16]="";
  int  ifnamelen;

#ifdef AGENT_DEBUG
  printf("getIfStats option=%s\n",ifname);
#endif


  deallocOldIfStats();


  fd=fopen(_PROC_NET_DEV,"r");
  if (fd!=NULL){
 
    fgets(buf,BUFLEN,fd); /* dump Title line */
    fgets(buf,BUFLEN,fd); /* dump Title line */
    while(fgets(buf,BUFLEN,fd)!=NULL){

      /* compensate empty lines in RH9 Linux Kernel 2.4.20, in 2.4.6, SUSE 7.0 
	 there are no empty lines in /proc/net/route */
      if (strlen(buf)<30) continue; 

      bufptr=strstr(buf,":");
      
      bufptr+=sizeof(char);
      ifnamelen=bufptr-buf-1;
      if (ifnamelen>16) {warning("interfaces.c","ifnamelen > 16, adapt getIfStats");ifnamelen=16;}
      strncpy(tmp,buf,ifnamelen);
      sscanf(tmp,"%16s",iface);

#ifdef AGENT_DEBUG
      printf("IFNAME=*|%s|*\n",iface);
#endif

      if ((ifname==NULL) || (strcmp(iface,ifname)==0) || (strcmp(ifname,"")==0)) {
	if (ifStat==NULL){
	  ++ifStat_size;
	  ifStat=(struct nsif__interfaceStats *) calloc(1,sizeof(struct nsif__interfaceStats));
	}else{
	  ++ifStat_size;
	  ifStat=(struct nsif__interfaceStats *) realloc(ifStat,ifStat_size*sizeof(struct nsif__interfaceStats));
	}
	if (ifStat==NULL) error("interfaces.c:getIfStats","Not enough Memory",-2);
	
	ifStat[ifStat_size-1].ifname=(xsd__string) malloc(strlen(iface)+1);
	strncpy(ifStat[ifStat_size-1].ifname,iface,strlen(iface)+1); 


	sscanf(bufptr,"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",&ifStat[ifStat_size-1].rx_bytes,&ifStat[ifStat_size-1].rx_packets,&ifStat[ifStat_size-1].rx_errors,&ifStat[ifStat_size-1].rx_drop,&ifStat[ifStat_size-1].rx_fifo,&ifStat[ifStat_size-1].rx_frame,&ifStat[ifStat_size-1].rx_compressed,&ifStat[ifStat_size-1].rx_multicast,&ifStat[ifStat_size-1].tx_bytes,&ifStat[ifStat_size-1].tx_packets,&ifStat[ifStat_size-1].tx_errors,&ifStat[ifStat_size-1].tx_drop,&ifStat[ifStat_size-1].tx_fifo,&ifStat[ifStat_size-1].tx_colls,&ifStat[ifStat_size-1].tx_carrier,&ifStat[ifStat_size-1].tx_compressed);
      }
    }
  }else
    perror("getIfStats:Fehler");
  
  fclose(fd);
#ifdef AGENT_DEBUG
  printf("ifStats ENDE Size=%d ptr=%p\n",ifStat_size,ifStat);
#endif
  return;
  

}
