
// Webservice Device-Management 
// AGENT, route.c
// Author: Oliver Bohle o.bohle@tu-bs.de
// Technische Universitt Braunschweig
#include "agent.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include "../headers/SOAPdevmgrStub.h" 
#include <arpa/inet.h>


#define BUFLEN 100
#define FLAGSTRINGSIZE 25



char* procHexToCharIP(char *instr,int strlen){
  int hex;
  char buff[strlen+2];
  struct in_addr in,*ptr;
  
  ptr=&in;
  bzero(buff,strlen+2);
  strcat(buff,"0x");
  strcat(buff,instr);
  sscanf(buff,"%X",&hex);
  in.s_addr=hex;
  return inet_ntoa(in);
}


int procHexToInt(char *instr,int strlen){
  int hex;
  char buff[strlen+2];
  struct in_addr in,*ptr;
  
  ptr=&in;
  bzero(buff,strlen+2);
  strcat(buff,"0x");
  strcat(buff,instr);
  sscanf(buff,"%X",&hex);
  return hex;
}


void freeRoutingInformation(struct nsrt__routingInformation *ri){
  int j;

    if (ri->dst_addr!=NULL){
      free(ri->dst_addr->addr);
      free(ri->dst_addr);
    }
    if (ri->gw_addr!=NULL){
      free(ri->gw_addr->addr);
      free(ri->gw_addr);
    }
    if (ri->nm_addr!=NULL){
      free(ri->nm_addr->addr);
      free(ri->nm_addr);
    }
    free(ri->iface);
    for (j=0;j<ri->__sizeflags;j++) free(ri->flag[j]);
    free(ri->flag);
}



// FLAGS out of route.c in net-tools 
struct flagContainer * getRouteFlags(int iflags,struct flagContainer *rfc){
  
  int ctr=0;
  char **strarr;
  strarr=NULL;

  if (iflags & RTF_UP){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "UP", AGENT_FLAGLEN);
  }
  if (iflags & RTF_GATEWAY){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "GATEWAY", AGENT_FLAGLEN);
  }
  if (iflags & RTF_REJECT){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "REJECT", AGENT_FLAGLEN);
  }   
  if (iflags & RTF_HOST){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "HOST", AGENT_FLAGLEN);
  }    
  if (iflags & RTF_REINSTATE){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "REINSTATE", AGENT_FLAGLEN);
  }
  if (iflags & RTF_DYNAMIC){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "DYNAMIC", AGENT_FLAGLEN);
  }
  if (iflags & RTF_MODIFIED){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "MODIFIED", AGENT_FLAGLEN);
  }
  if (iflags & RTF_DEFAULT){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "DEFAULT", AGENT_FLAGLEN);
  }
  if (iflags & RTF_ALLONLINK){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "ALLONLINK", AGENT_FLAGLEN);
  }
  if (iflags & RTF_ADDRCONF){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "ADDRCONF", AGENT_FLAGLEN);
  }
  if (iflags & RTF_NONEXTHOP){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "NONEXTHOP", AGENT_FLAGLEN);
  }
  if (iflags & RTF_EXPIRES){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "EXPIRES", AGENT_FLAGLEN);
  }
  if (iflags & RTF_CACHE){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "CACHE", AGENT_FLAGLEN);
  }
  if (iflags & RTF_FLOW){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "FLOW", AGENT_FLAGLEN);
  }
  if (iflags & RTF_POLICY){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "POLICY", AGENT_FLAGLEN);
  }
  if (iflags & RTF_LOCAL){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "LOCAL", AGENT_FLAGLEN);
  }
  if (iflags & RTF_MTU){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "MTU", AGENT_FLAGLEN);
  }
  if (iflags & RTF_WINDOW){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "WINDOW", AGENT_FLAGLEN);
  }
  if (iflags & RTF_IRTT){
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "IRTT", AGENT_FLAGLEN);
  }
  if (iflags & RTF_NOTCACHED){             /* 2.0.36 */
    strarr=(char**) realloc(strarr,++ctr*sizeof(char*));
    strarr[ctr-1]=(char *) calloc(AGENT_FLAGLEN,sizeof(char));
    strncat(strarr[ctr-1], "NOTCACHED", AGENT_FLAGLEN);
  }

  rfc->array=strarr;
  rfc->size=ctr;

  return rfc;
}




/* returns the integer-representation of the Flags stored in the ri struct */
int setRouteFlags(struct nsrt__routingInformation *ri){
  int i,flagval=0;

  for(i=0;i<ri->__sizeflags;i++){
    if (strcmp(ri->flag[i],"UP")==0) {flagval+= RTF_UP;continue;}
    if (strcmp(ri->flag[i],"GATEWAY")==0) {flagval+= RTF_GATEWAY;continue;}
    if (strcmp(ri->flag[i],"REJECT")==0) {flagval+= RTF_REJECT;continue;}
    if (strcmp(ri->flag[i],"HOST")==0) {flagval+= RTF_HOST;continue;}
    if (strcmp(ri->flag[i],"REINSTATE")==0) {flagval+= RTF_REINSTATE;continue;}
    if (strcmp(ri->flag[i],"DYNAMIC")==0) {flagval+= RTF_DYNAMIC;continue;}
    if (strcmp(ri->flag[i],"MODIFIED")==0) {flagval+= RTF_MODIFIED;continue;}
    if (strcmp(ri->flag[i],"DEFAULT")==0) {flagval+= RTF_DEFAULT;continue;}
    if (strcmp(ri->flag[i],"ALLONLINK")==0) {flagval+= RTF_ALLONLINK;continue;}
    if (strcmp(ri->flag[i],"ADDRCONF")==0) {flagval+= RTF_ADDRCONF;continue;}
    if (strcmp(ri->flag[i],"NONEXTHOP")==0) {flagval+= RTF_NONEXTHOP;continue;}
    if (strcmp(ri->flag[i],"EXPIRES")==0) {flagval+= RTF_EXPIRES;continue;}
    if (strcmp(ri->flag[i],"CACHE")==0) {flagval+= RTF_CACHE;continue;}
    if (strcmp(ri->flag[i],"FLOW")==0) {flagval+= RTF_FLOW;continue;}
    if (strcmp(ri->flag[i],"POLICY")==0) {flagval+= RTF_POLICY;continue;}
    if (strcmp(ri->flag[i],"LOCAL")==0) {flagval+= RTF_LOCAL;continue;}
    if (strcmp(ri->flag[i],"MTU")==0) {flagval+= RTF_MTU;continue;}
    if (strcmp(ri->flag[i],"WINDOW")==0) {flagval+= RTF_WINDOW;continue;}
    if (strcmp(ri->flag[i],"IRTT")==0) {flagval+= RTF_IRTT;continue;}
    if (strcmp(ri->flag[i],"NOTCACHED")==0) {flagval+= RTF_NOTCACHED;continue;}
  }
  return flagval;
  
}



/* determines how the flag changed (del,new) and adds the value in del
   or new if a differnce is found.
   Returns 1 if a difference exist else 0.  */ 
int checkFlagChange(int oldflag,int newflag,int flag,int *new,int *del){

  if (oldflag & flag & newflag) return 0;
  if (oldflag & flag) {(*del)+=flag;return 1;}
  if (newflag & flag) {(*new)+=flag;return 1;}
  return -1;
}



/* checks if the Flags in iflags are equal to the flags in ri. If
   true, the return value is 1 else 0. rfc shall be a
   routeFlagsContainer array with two entrys. in the first one the
   deleted flags are set, in the second one the new ones. */

int equalFlags(int newflags,struct nsrt__routingInformation *ri,struct flagContainer *rfc){
  
   int oldflags;
   int new=0, del=0;


  
   /* comparing to find differnt flags */
   oldflags=setRouteFlags(ri);

   if (newflags==oldflags) return 1;

   /* FLAGS are different */
   
   checkFlagChange(oldflags,newflags,RTF_UP,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_GATEWAY,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_REJECT,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_HOST,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_REINSTATE,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_DYNAMIC,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_MODIFIED,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_DEFAULT,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_ALLONLINK,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_ADDRCONF,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_NONEXTHOP,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_EXPIRES,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_CACHE,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_FLOW,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_POLICY,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_LOCAL,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_MTU,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_WINDOW,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_IRTT,&new,&del);
   checkFlagChange(oldflags,newflags,RTF_NOTCACHED,&new,&del);
   
   if (del!=0) getRouteFlags (del,&rfc[0]);
   if (new!=0) getRouteFlags (new,&rfc[1]);

   return 0;
}




/* parse the /proc/net/route entry and assign the values to the matching fields 
   in the nsrt__routingInformation struct */


void parseRoutingData(struct nsrt__routingInformation *ri,char *buf){
  char iface[16];
  char gate_addr[128];
  char mask_addr[128], dest_addr[128];
  int  iflags;
  struct flagContainer rfc;

  sscanf(buf,"%16s %128s %128s %X %d %d %d %128s %d %d %d",iface,dest_addr,gate_addr,&iflags,&ri->refcnt,&ri->use,&ri->metric,mask_addr,&ri->mtu,&ri->window,&ri->irtt);

  ri->iface=(xsd__string) malloc(strlen(iface)+1);
  strncpy(ri->iface,iface,strlen(iface)+1);
      
  ri->dst_addr=(struct ns__address *) malloc(sizeof(struct ns__address));
  ri->dst_addr->type=IP;
  ri->dst_addr->addr=(char *) calloc(AGENT_IPLEN,sizeof(char));
  strncat(ri->dst_addr->addr,procHexToCharIP(dest_addr,128),AGENT_IPLEN);
  
  ri->gw_addr=(struct ns__address *) malloc(sizeof(struct ns__address));
  ri->gw_addr->type=IP;
  ri->gw_addr->addr=(char *) calloc(AGENT_IPLEN,sizeof(char));
  strncat(ri->gw_addr->addr,procHexToCharIP(gate_addr,128),AGENT_IPLEN);
  
  ri->nm_addr=(struct ns__address *) malloc(sizeof(struct ns__address));
  ri->nm_addr->type=IP;
  ri->nm_addr->addr=(char *) calloc(AGENT_IPLEN,sizeof(char));
  strncat(ri->nm_addr->addr,procHexToCharIP(mask_addr,128),AGENT_IPLEN);
  
  getRouteFlags(iflags,&rfc);
  ri->flag=rfc.array;
  ri->__sizeflags=rfc.size;
} 




/* comparing /proc/net/route entry with the existing routing
   information in rtInf returns the elementposition of the element
   with the same netmask,interface name, destiantion and gateway
   address,-1 if no element is found.
 */ 
int findSimilarRoutingData(char *buf){
   char iface[16];
  char gate_addr[128];
  char mask_addr[128], dest_addr[128];
  int  iflags, metric;
  int  mtu,win,irtt,refcnt,use;
  int i;

  sscanf(buf,"%16s %128s %128s %X %d %d %d %128s %d %d %d",iface,dest_addr,gate_addr,&iflags,&refcnt,&use,&metric,mask_addr,&mtu,&win,&irtt);

  for(i=0;i<rtInf_size;i++){
    if (strcmp(rtInf[i].iface,iface)==0) 
      if (inet_addr(rtInf[i].dst_addr->addr)==procHexToInt(dest_addr,128))
	if (inet_addr(rtInf[i].nm_addr->addr)==procHexToInt(mask_addr,128)) 
	  if (inet_addr(rtInf[i].gw_addr->addr)==procHexToInt(gate_addr,128)) 
	    return i;
  }
  return -1;
}





/* Checks if the routing information of a route have changed.
   The Information netmask,interface name, destiantion and gateway
   address are not checked due to the preceeding check by findSimilarRoutingData().
   If a difference is found the return value is 1 else 0 */

int updateChangedRoutingData(struct nsrt__routingInformation *ri,char *buf){
  char iface[16];
  char gate_addr[128];
  char mask_addr[128], dest_addr[128];
  int  iflags, metric=0;
  int  mtu,win,irtt,refcnt,use;
  int found=0;
  struct flagContainer rfc[2],updrfc;
  char msg[AGENT_ERRMSGLEN];
  int ret;


  rfc[0].size=0;
  rfc[0].array=NULL;
  rfc[1].size=0;
  rfc[1].array=NULL;


  bzero(msg,sizeof(msg));
  sprintf(msg,"CHANGED ROUTE ENTRY :: IFNAME=%s DEST_IP=%s GATEWAY=%s NETMASK=%s ::",rtInf[rtInf_size-1].iface,rtInf[rtInf_size-1].dst_addr->addr,rtInf[rtInf_size-1].gw_addr->addr,rtInf[rtInf_size-1].nm_addr->addr);
  ret=EVT_TP_ROUTE;
  

  sscanf(buf,"%16s %128s %128s %X %d %d %d %128s %d %d %d",iface,dest_addr,gate_addr,&iflags,&refcnt,&use,&metric,mask_addr,&mtu,&win,&irtt);
   
  
  if (refcnt!=ri->refcnt) {found=1; sprintf(msg,"%s REFCNT %d-->%d\n",msg,ri->refcnt,refcnt);ri->refcnt=refcnt;}
  if (use!=ri->use) {found=1;sprintf(msg,"%s USE %d-->%d\n",msg,ri->use,use);ri->use=use;}
  if (metric!=ri->metric) {found=1;sprintf(msg,"%s METRIC %d-->%d\n",msg,ri->metric,metric);ri->metric=metric;}
  if (mtu!=ri->mtu) {found=1; sprintf(msg,"%s MTU %d-->%d\n",msg,ri->mtu,mtu);ri->mtu=mtu;}
  if (win!=ri->window) {found=1;sprintf(msg,"%s WIN %d-->%d\n",msg,ri->window,win);ri->window=win;}
  if (irtt!=ri->irtt) {found=1;sprintf(msg,"%s IRTT %d-->%d\n",msg,ri->irtt,irtt);ri->irtt=irtt;}
  if (!(equalFlags(iflags,ri,rfc))) {
    found=1;
    getRouteFlags(iflags,&updrfc);
    free(ri->flag);
    ri->flag=updrfc.array;
    ri->__sizeflags=updrfc.size;
  }

  if (rfc[0].size>0){
    int i=0;
    sprintf(msg,"%s DELETED FLAGS = <",msg);
    for (i=0;i<rfc[0].size;i++) sprintf(msg,"%s %s",msg,rfc[0].array[i]);
    sprintf(msg,"%s >",msg);
  }
  if (rfc[1].size>0){
    int i=0;
    sprintf(msg,"%s ADDED FLAGS = <",msg);
    for (i=0;i<rfc[1].size;i++) sprintf(msg,"%s %s",msg,rfc[1].array[i]);
    sprintf(msg,"%s >",msg);
  }
  
  
  if (found) sendEventMessage(msg,ret );	
  return found;
}




/* get information of /proc/net/route for routing-information and store them in rfInf*/ 

void create_routelist_by_Proc_net_route(){
  FILE *fd;
  char buf[BUFLEN]="";
  int count=0;


  fd=fopen(_PROC_NET_ROUTE,"r");
  if (fd!=NULL){
 
    fgets(buf,BUFLEN,fd); /* dump Title line */
    
    while(fgets(buf,BUFLEN,fd)!=NULL){

      /* compensate empty lines in RH9 Linux Kernel 2.4.20, in 2.4.6, SUSE 7.0 
	 there are no empty lines in /proc/net/route */
      if (strlen(buf)<30) continue; 

 
      if (rtInf==NULL){
	++rtInf_size;
	rtInf=(struct nsrt__routingInformation *) calloc(1,sizeof(struct nsrt__routingInformation));
      }else{
	++rtInf_size;
	rtInf=(struct nsrt__routingInformation *) realloc(rtInf,rtInf_size*sizeof(struct nsrt__routingInformation));
      }
      if (rtInf==NULL) error("create_interface_list","Not enough Memory",-2);
      parseRoutingData(&rtInf[count],buf);
      count++;
    }
  }else
    perror("readProc:Fehler");
  
  fclose(fd);  
  //printf("READY WITH FETCHING ROUTE INFORMATION, created %d entrys\n",rtInf_size);
  return;
}






void update_RoutingInformation(){  
  FILE *fd;
  char buf[BUFLEN]="";
  int count=0,i=0;
  int result,changed=0;
  int match[rtInf_size];              /* array to check, if an entry was removed */
  int match_size=0,todelete=0,tmp_size=0;
  struct nsrt__routingInformation *ritmp;
 
#ifdef AGENT_DEBUG 
  printf("UPDATING ROUTINGINFORMATION\n");
#endif

  if  (rtInf==NULL) {
    create_routelist_by_Proc_net_route();
    return;
  }

  match_size=rtInf_size;
  for(i=0;i<rtInf_size;i++) match[i]=0;


  fd=fopen(_PROC_NET_ROUTE,"r");
  if (fd!=NULL){
    fgets(buf,BUFLEN,fd); /* dump Title line */
    while(fgets(buf,BUFLEN,fd)!=NULL){

      /* compensate empty lines in RH9 Linux Kernel 2.4.20, in 2.4.6, SUSE 7.0 
	 there are no empty lines in /proc/net/route */
      if (strlen(buf)<30) continue;

      /* find existing routes in the routing table and check for new values */
      result=findSimilarRoutingData(buf);
      if (result!=-1){
	match[result]=1;                                            /* mark entry as found */
	changed=0;
	changed=updateChangedRoutingData(&rtInf[result],buf);       /* changed entry since last update ? */
	if (changed) printf ("ROUTING DATA CHANGED\n");             
      }
      else{
	 char msg[AGENT_ERRMSGLEN]="";
	 int ret;
	/* new routing entry found, append it to rtInf */
	if (rtInf==NULL){
	  ++rtInf_size;
	  rtInf=(struct nsrt__routingInformation *) calloc(1,sizeof(struct nsrt__routingInformation));
	}else{
	  ++rtInf_size;
	  rtInf=(struct nsrt__routingInformation *) realloc(rtInf,rtInf_size*sizeof(struct nsrt__routingInformation));
	}
	if (rtInf==NULL) error("update_RoutingInformation","Not enough Memory",-2);
	parseRoutingData(&rtInf[rtInf_size-1],buf);

	/* SEND EVENTMESSAGE */
	bzero(msg,sizeof(msg));
	sprintf(msg,"NEW ROUTE ENTRY :: IFNAME=%s DEST_IP=%s GATEWAY=%s NETMASK=%s",rtInf[rtInf_size-1].iface,rtInf[rtInf_size-1].dst_addr->addr,rtInf[rtInf_size-1].gw_addr->addr,rtInf[rtInf_size-1].nm_addr->addr);
	ret=EVT_TP_ROUTE;
	sendEventMessage(msg,ret );	
      }
      count++;
    }

    /* check for deleted entrys */ 
    for (i=0;i<match_size;i++) {
      if (!match[i]) todelete++;
    }
    if (todelete){
      /* delete entrys */
      ritmp=rtInf;
      tmp_size=rtInf_size;
      rtInf=(struct nsrt__routingInformation *) calloc(rtInf_size-todelete,sizeof(struct nsrt__routingInformation));
      if (rtInf==NULL) error("update_RoutingInformation","Not enough Memory",-2);
      rtInf_size=rtInf_size-todelete;
      count=0;
      for (i=0;i<tmp_size;i++) {
	char msg[AGENT_ERRMSGLEN]="";
	int ret;
	if (i<match_size) {
	  if (match[i]) {
	    rtInf[count++]=ritmp[i];
	  }else{
	    /* DELETED ENTRY FOUND */	   
	    bzero(msg,sizeof(msg));
	    sprintf(msg,"DELETED ROUTE ENTRY :: IFNAME=%s DEST_IP=%s GATEWAY=%s NETMASK=%s",ritmp[i].iface,ritmp[i].dst_addr->addr,ritmp[i].gw_addr->addr,ritmp[i].nm_addr->addr);
	    ret=EVT_TP_ROUTE;
	    sendEventMessage(msg,ret );
	    freeRoutingInformation(&ritmp[i]);
	  }

	}else{
	  rtInf[count++]=ritmp[i];
	}	
      }
      /* free ritmp */
      free(ritmp);
    }
    
  }else
    perror("readProc:Fehler");
  

  fclose(fd);
  return;
  
}
